#!/usr/bin/env python
#-*- coding:utf-8 -*-

# Copyright (C) 2010 Antony Templier
# 
# This software is under WTFPL License:
# ------------------------------------------------------------------------------
#             DO WHAT THE FUCK YOU WANT TO PUBLIC LICENSE
#                     Version 2, December 2004
# 
#  Copyright (C) 2004 Sam Hocevar <sam@hocevar.net>
# 
#  Everyone is permitted to copy and distribute verbatim or modified
#  copies of this license document, and changing it is allowed as long
#  as the name is changed.
# 
#             DO WHAT THE FUCK YOU WANT TO PUBLIC LICENSE
#    TERMS AND CONDITIONS FOR COPYING, DISTRIBUTION AND MODIFICATION
# 
#   0. You just DO WHAT THE FUCK YOU WANT TO.
# ------------------------------------------------------------------------------

# based on 0423.2000 by michal wallace http://www.sabren.com/
# based on perl's Crypt::PasswdMD5 by Luis Munoz (lem@cantv.net)
# based on /usr/src/libcrypt/crypt.c from FreeBSD 2.2.5-RELEASE
#
# It's based on the implementation found on FreeBSD 2.2.[56]-RELEASE and
# contains the following license in it:
# 
#  "THE BEER-WARE LICENSE" (Revision 42):
#  <phk@login.dknet.dk> wrote this file.  As long as you retain this notice you
#  can do whatever you want with this stuff. If we meet some day, and you think
#  this stuff is worth it, you can buy me a beer in return.   Poul-Henning Kamp
# 
# Michal Wallace add MANY THANKS TO
#  Carey Evans - http://home.clear.net.nz/pages/c.evans/
#  Dennis Marti - http://users.starpower.net/marti1/
#  For the patches that got this thing working!

# apache_md5_crypt() provides a function compatible with Apache's
# .htpasswd files. This was contributed by Bryan Hart <bryan@eai.com>.
#
# Antony Templier contributed by adding the following functions :
# __get_salt() provides a function to return a random 64bits integer from
# urandom.
# encrypt() & check_password(), both just for easy utilisation.

"""
Provides interoperable MD5-based crypt() function

unix_md5_crypt() provides a crypt()-compatible interface to the
rather new MD5-based crypt() function found in modern operating systems.
"""

__version__ = '1.0.0'
__author__ = "Antony Templier <antony.tempier@gmail.com>"
__date__ = "30 june 2011"
__license__ = "WTFPL"
__credits__ = "Just adding function to  Michal Wallace script, base on Luis Munoz code, base on Poul-Henning Kamp code."

import hashlib
from os import urandom
from binascii import hexlify

MAGIC = '$1$'			# Magic string
ITOA64 = "./0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz"

def __get_salt():
    # int(binascii.hexlify(os.urandom(8)), 16) is a 64bit integer
    return to64(int(hexlify(urandom(8)), 16), 8)

def to64 (v, n):
    """Convert int to ascii base64
    
    @type   v: Int
    @param  v: numbr to convert

    @type   n: Int
    @param  n: numer of column to display
    
    @rtype  String
    @rparam 64bits version of the number

    """ 
    ret = ''
    while (n - 1 >= 0):
        n = n - 1
	ret = ret + ITOA64[v & 0x3f]
	v = v >> 6
    return ret

def apache_md5_crypt (pw, salt):
    # change the Magic string to match the one used by Apache
    return unix_md5_crypt(pw, salt, '$apr1$')

def unix_md5_crypt(pw, salt, magic=None):
    
    if magic==None:
        magic = MAGIC

    # Take care of the magic string if present
    if salt[:len(magic)] == magic:
        salt = salt[len(magic):]
        

    # salt can have up to 8 characters:
    import string
    salt = string.split(salt, '$', 1)[0]
    salt = salt[:8]

    ctx = pw + magic + salt

    final = hashlib.md5(pw + salt + pw).digest()

    for pl in range(len(pw),0,-16):
        if pl > 16:
            ctx = ctx + final[:16]
        else:
            ctx = ctx + final[:pl]


    # Now the 'weird' xform (??)

    i = len(pw)
    while i:
        if i & 1:
            ctx = ctx + chr(0)  #if ($i & 1) { $ctx->add(pack("C", 0)); }
        else:
            ctx = ctx + pw[0]
        i = i >> 1

    final = hashlib.md5(ctx).digest()
    
    # The following is supposed to make
    # things run slower. 

    # my question: WTF???

    for i in range(1000):
        ctx1 = ''
        if i & 1:
            ctx1 = ctx1 + pw
        else:
            ctx1 = ctx1 + final[:16]

        if i % 3:
            ctx1 = ctx1 + salt

        if i % 7:
            ctx1 = ctx1 + pw

        if i & 1:
            ctx1 = ctx1 + final[:16]
        else:
            ctx1 = ctx1 + pw
            
            
        final = hashlib.md5(ctx1).digest()


    # Final xform
                                
    passwd = ''

    passwd = passwd + to64((int(ord(final[0])) << 16)
                           |(int(ord(final[6])) << 8)
                           |(int(ord(final[12]))),4)

    passwd = passwd + to64((int(ord(final[1])) << 16)
                           |(int(ord(final[7])) << 8)
                           |(int(ord(final[13]))), 4)

    passwd = passwd + to64((int(ord(final[2])) << 16)
                           |(int(ord(final[8])) << 8)
                           |(int(ord(final[14]))), 4)

    passwd = passwd + to64((int(ord(final[3])) << 16)
                           |(int(ord(final[9])) << 8)
                           |(int(ord(final[15]))), 4)

    passwd = passwd + to64((int(ord(final[4])) << 16)
                           |(int(ord(final[10])) << 8)
                           |(int(ord(final[5]))), 4)

    passwd = passwd + to64((int(ord(final[11]))), 2)


    return magic + salt + '$' + passwd

def check_password(clearpw, hashpw):
    """
    To check if both password matches
    
    @type   clearpw: String
    @param  clearpw: non encrypted password

    @type   hash: String
    @param  hash: encrypted version of password done with md5crypt function

    @rtype  Boolean
    @rparam True if both passwords matche
    """

    result = False
    magic, salt = hashpw[1:].split('$')[:2]
    magic = '$' + magic + '$'
    res = md5crypt(clearpw, salt, magic) 

    if res==hashpw:
        result = True
        
    return result

def encrypt(password):
    return md5crypt(password, __get_salt())

## assign a wrapper function:
md5crypt = unix_md5_crypt

